@@ -20,7 +20,7 @@
 #'@export
 
 realized <- function(zres, classNum = NULL) {
-  K = length(zres@krates[1,])
+  K = length(zres@krates[1,])+1
   if(is.null(classNum)){
     classNum = seq(1,K)
   }
@@ -28,12 +28,12 @@ realized <- function(zres, classNum = NULL) {
   K2 <- length(classNum)
   classNames <- array('',K2)
   for (i in 1:K2){
-    if(sd(zres@krates[,K])!=0){
-      classNames[i]=paste("HBDclass",classNum[i],sep="")
+    if(sd(zres@krates[,K-1])!=0){
+      if(classNum[i] < K){classNames[i]=paste("HBDclass",classNum[i],sep="")}
       if(classNum[i]==K){classNames[i]="NonHBD"}
     }
-    if(sd(zres@krates[,K])==0){
-      classNames[i]=paste("R_",zres@krates[1,classNum[i]],sep="")
+    if(sd(zres@krates[,K-1])==0){
+      if(classNum[i] < K){classNames[i]=paste("R_",zres@krates[1,classNum[i]],sep="")}
       if(classNum[i]==K){classNames[i]="NonHBD"}
     }
   }
@@ -58,8 +58,8 @@ realized <- function(zres, classNum = NULL) {
 #'@export
 
 cumhbd <- function(zres, T = NULL) {
-  K = length(zres@krates[1,])
-  if(sd(zres@krates[,K])!=0){
+  K = length(zres@krates[1,])+1
+  if(sd(zres@krates[,K-1])!=0){
     print(c("cumhbd is not recommended for a model with different rates per individual!"))
   }
   nind = length(zres@ids)
@@ -176,7 +176,7 @@ rohbd <- function(zres, ids = NULL, chrom = NULL, startPos = NULL, endPos = NULL
 
 probhbd <- function(zres, zooin, id, chrom = NULL, startPos = NULL, endPos = NULL, T = FALSE) {
   hbdp <- array(0,zooin@nsnps)
-  K=length(zres@krates[id,]);K2=K-1
+  K=length(zres@krates[id,])+1;K2=K-1
   if(T){
     K2=0
     for (k in 1:(K-1)){if(zres@krates[id,k] <= T){K2=k}}
@@ -205,128 +205,116 @@ probhbd <- function(zres, zooin, id, chrom = NULL, startPos = NULL, endPos = NUL
   return(hbdp)
 }
 
-#'Returns number of HBD segments (or genome proportion) per size bins for DoRIS.
+#'Merge several zres objects generated by zoorun
 #'
-#'The number of HBD segments in 1Mb bins is computed and the function returns a
-#'table that can be used with DoRIS to estimate paste effective population size.
-#'Alternatively, the proportion of the genome in the different 1 Mb bins is
-#'computed. The user must specify the range of the bins (the smallest and largest
-#'HBD segments considered).
-#'#'
-#'@param zres The name of the zres object created by the zoorun function.
+#'The function is used for example when the analysis has been split in several
+#'tasks. All the zres must come from the same original
+#'zoodata (zoorun is applied to the same zoodata with the same model). In
+#'addition, the data should not overlap (the same individual should not be
+#'present in multiple zres).
 #'
-#'@param minv The minimum length of HBD segments (in Mb) used for the DoRIS analysis.
+#'@param list_zres A list with the name of the zres objects to be merged.
 #'
-#'@param maxv The maximum length of HBD segments (in Mb) used for the DoRIS analysis.
-#'
-#'@param glen The length of the genome in Mb (including only the autosomes, the space where
-#'HBD segments have been searched for).
-#'
-#'@param method The argument is used to indicate that function will return counts of
-#'HBD segments (method = "counts") or proportion of the genome (method ="sharing") per
-#'1 Mb bins.
-#'
-#'@return A data frame with three columns and one row per 1Mb bins. The first column
-#'indicates the the length of smallest HBD segments in the bin, the second column
-#'indicates the length of the largest HBD segments in the bin and the third column
-#'indicates the number of segments in the bin or the proportion of the genome
-#'in the bin.
+#'@return a single zres object containing the results from the merged zres
+#'  objects. Note that the HBD segments are not sorted by ID number.
 #'
 #'@export
 
-zoodoris <- function(zres, minv, maxv, glen, method='counts') {
+merge_zres <- function(list_zres){
 
-win_st <- seq(minv,(maxv-1))
-win_end <- seq((minv+1),maxv)
+  allres <- new("zres")
 
-nseg = array(0,length(win_st))
-sharing = array(0,length(win_st))
-npairs = zres@nind
+  allres@nind <- list_zres[[1]]@nind+list_zres[[2]]@nind
+  allres@ids <- c(list_zres[[1]]@ids,list_zres[[2]]@ids)
+  allres@mixc <- rbind(list_zres[[1]]@mixc,list_zres[[2]]@mixc)
+  allres@krates <- rbind(list_zres[[1]]@krates,list_zres[[2]]@krates)
+  allres@realized <- rbind(list_zres[[1]]@realized,list_zres[[2]]@realized)
+  allres@modlik <- c(list_zres[[1]]@modlik,list_zres[[2]]@modlik)
+  allres@modbic <- c(list_zres[[1]]@modbic,list_zres[[2]]@modbic)
+  allres@niter <- c(list_zres[[1]]@niter,list_zres[[2]]@niter)
+  allres@optimerr <- c(list_zres[[1]]@optimerr,list_zres[[2]]@optimerr)
+  allres@sampleids <- c(list_zres[[1]]@sampleids,list_zres[[2]]@sampleids)
+  allres@hbdp <- c(list_zres[[1]]@hbdp,list_zres[[2]]@hbdp) ### stays a list
+  allres@hbdseg <- rbind(list_zres[[1]]@hbdseg,list_zres[[2]]@hbdseg)
 
-for (i in 1:length(nseg)){
-sel <- zres@hbdseg[zres@hbdseg$length/1000000 > win_st[i] &
-                     zres@hbdseg$length/1000000 <= win_end[i],]
-nseg[i] <- dim(sel)[1]
-sharing[i] <- sum(sel$length/1000000) /(glen*npairs)
-}
-
-out1 <- data.frame(cbind(win_st, win_end, nseg))
-out2 <- data.frame(cbind(win_st, win_end, sharing))
-
-if(method=='counts'){return(out1)}
-if(method=='sharing'){return(out2)}
-
-}
-
-#'Realizes a simulation using a zooroh data set and a zooroh model
-#'
-#'Performs a simulation under a model similar too ZooRoH. It simulates the
-#'genome as a mosaic of HBD and non-HBD segments. Several non-HBD classes can be
-#'simulated. Classes with high rates (short segments from ancient ancestors) are
-#'simulated first. Then, more recent classes are subsequently added. Recent HBD
-#'segments will mask more ancient HBD segments. See details in the method
-#'published in Molecular Ecology (Druet and Gautier, 2017). Genotypes are
-#'simulated using provided allele frequencies (from the sample), their genetic
-#'distances, the number of chromosomes and the number of SNPs. These simulations
-#'do not take into account linkage disequilibrium information. This simulation
-#'tool can for instance be used to check whether there is enough information in
-#'the data set to estimate HBD segments and their partitioning in multiple
-#'classes. Note that this simulation tool is not computationally efficient.
-#'
-#'@param simdata The name of the zdata object created by the zoodata function.
-#'
-#'@param simmodel The name of the zmodel object created by the zoomodel
-#'  function. The simulation program uses only the number of classes, their rate
-#'  and their mixing coefficient (it is the same for a pre-defined model or
-#'  not).
-#'
-#'@param nsim The number of simulated individuals.
-#'
-#'@param fullout Indicates whether a more detailed output is requested.
-#'
-#'@return The function simulates genotypes with the properties of the data set
-#'  (number of SNPs, genetic map, allele frequencies) and according to the
-#'  specified model (number of HBD classes, rate of the classes and proportion
-#'  of mixing). The simulation is created as in Druet and Gautier (2017) and new
-#'  autozygosity masks more ancient autozygosity. The output is a new zoodata
-#'  object that can be analyzed with the zoorun function.
-#'
-#'  If a more detailed output is requested with the fullout parameter, then the
-#'  function returns list (for instance simres) containing the zoodata, a matrix
-#'  with realized inbreeding per individual, estimated at SNP positions, in the
-#'  different classes (with 1 being the most ancient class (non-HBD) and the
-#'  highest number corresponds to the most recent class), and a matrix
-#'  containing for each individual and at each marker position the simulated
-#'  class (1 for non-HBD, 2 for most ancient HBD, etc). These elements can be
-#'  accessed using the simres[[1]], simres[[2]] and simres[[3]], respectively.
-#'@export
-
-zoosimd <- function(simdata, simmodel, nsim, fullout = FALSE) {
-
-  nlayers = length(simmodel@krates)-1
-  runsim <- .Fortran("zoosim",as.integer(nlayers),as.integer(simdata@nchr),as.integer(simdata@nsnps),
-                 as.double(simdata@freqs),as.integer(simdata@bp),
-                 as.integer(simdata@chrbound),as.double(simmodel@krates[nlayers:1]),
-                 as.double(simmodel@mix_coef[nlayers:1]),as.double(simmodel@err),as.integer(nsim),
-                 matrix(as.integer(0),nsim,simdata@nsnps),
-                 matrix(as.double(0),nsim,nlayers),matrix(as.integer(0),nsim,simdata@nsnps))
-
-  simout <- new("zdata")
-  simout@genos <- t(runsim[[11]])
-  simout@bp <- simdata@bp
-  simout@chrnames <- simdata@chrnames
-  simout@chrbound <- simdata@chrbound
-  simout@nind <- nsim
-  simout@nsnps <- simdata@nsnps
-  freqs <- apply(simout@genos, 1, getfreq1)
-  simout@freqs <- freqs
-  simout@nchr <- simdata@nchr
-  simout@zformat = "gt"
-
-  if(!fullout)return(simout)
-  if(fullout){
-    simres=list(simout,runsim[[12]],t(runsim[[13]]))
-    return(simres)
+  if(length(list_zres)>2){
+    for (i in 3:length(list_zres)){
+      allres@nind <- allres@nind + list_zres[[i]]@nind
+      allres@ids <- c(allres@ids,list_zres[[i]]@ids)
+      allres@mixc <- rbind(allres@mixc,list_zres[[i]]@mixc)
+      allres@krates <- rbind(allres@krates,list_zres[[i]]@krates)
+      allres@realized <- rbind(allres@realized,list_zres[[i]]@realized)
+      allres@modlik <- c(allres@modlik,list_zres[[i]]@modlik)
+      allres@modbic <- c(allres@modbic,list_zres[[i]]@modbic)
+      allres@niter <- c(allres@niter,list_zres[[i]]@niter)
+      allres@optimerr <- c(allres@optimerr,list_zres[[i]]@optimerr)
+      allres@sampleids <- c(allres@sampleids,list_zres[[i]]@sampleids)
+      allres@hbdp <- c(allres@hbdp,list_zres[[i]]@hbdp) ### stays a list
+      allres@hbdseg <- rbind(allres@hbdseg,list_zres[[i]]@hbdseg)
+    }
   }
+
+  x <- order(allres@ids)
+
+  allres@ids <- allres@ids[x]
+  allres@mixc <- allres@mixc[x,]
+  allres@krates <- allres@krates[x,]
+  allres@realized <- allres@realized[x,]
+  allres@modlik <- allres@modlik[x]
+  allres@modbic <- allres@modbic[x]
+  allres@niter <- allres@niter[x]
+  allres@optimerr <- allres@optimerr[x]
+  allres@sampleids <- allres@sampleids[x]
+  allres@hbdp <- allres@hbdp[x] ### stays a list
+  ## allres@hbdseg
+
+  n1 <- length(allres@ids)
+  n2 <- length(unique(allres@ids))
+
+  if(n2 < n1){
+    print("Some duplicate IDs were present in the input files.")
+    print("The merged zres files should contain distinct individuals")
+  }
+
+  return(allres)
+
+}
+
+#'Update one main zres object with new results
+#'
+#'The function is used for example when the main analysis failed for one individual.
+#'The analysis is repeated for that individual with other parameters. The function
+#'can then be used to insert the new results in the main zres object. To avoid
+#'generating to large files, the updated zres object can take the same name as zres1
+#'doing zres1 <- update_zres(zres1,zres2).
+#'
+#'@param zres1 The main zres object that will be modified.
+#'
+#'@param zres2 The new zres object, with the new results that will be inserted in the
+#'main zres object.
+#'
+#'@return an updated zres object containing the results from zres1 updated by those from
+#'zres2.
+#'
+#'@export
+
+update_zres <- function(zres1,zres2){
+
+  x <- match(zres2@ids,zres1@ids)
+
+  zres1@mixc[x,] <- zres2@mixc
+  zres1@krates[x,] <- zres2@krates
+  zres1@realized[x,] <- zres2@realized
+  zres1@modlik[x] <- zres2@modlik
+  zres1@modbic[x] <- zres2@modbic
+  zres1@niter[x] <- zres2@niter
+  zres1@optimerr[x] <- zres2@optimerr
+  zres1@hbdp[x] <- zres2@hbdp
+
+  zres1@hbdseg <- zres1@hbdseg[!zres1@hbdseg$id %in% zres2@ids,]
+  zres1@hbdseg <- rbind(zres1@hbdseg,zres2@hbdseg)
+
+  return(zres1)
+
 }
 
